#include <iostream.h>
#include <math.h>

class Moment {

public:

     Moment()  {}
     Moment(float* dat,long n):data(dat),nset(n) 
     
           {if (nset <= 1) error("n must be at least 2 in moment");}                             

     float Mean();   	//returns the average of the data set.
     float Var();		//returns the variance of the data set.
     float Sdev();		//returns the standard deviation
     float Adev();		//returns the average deviation
     float Skew();		//returns the skewness
     float Kurt();		//returns the kurtosis
     
private:

     void error(char ErrorTest[]);
     long j;
     long nset;
     float* data;     
};

// The code to calculate these generators are in "Numerical Recipes in C The Art of Scientific Computing 
// Second Edition"

// On Ch 14-1 p.613


float Moment::Mean() {

     float s=0.0;
     for (j=0;j<nset;j++) s += data[j];
     return s/nset;
}     

float Moment::Var() {

     float v=0.0;
     float ep=0.0;
     float d=0.0;
     float mean=Mean();
     
     for (j=0;j < nset;j++) {
     	
     	d=data[j] - mean;
     	ep += d;
     	v += (d*d);
     }	
     	
     return (v-ep*ep/nset)/(nset-1);
}

float Moment::Sdev() {return sqrt(Var());}

float Moment::Adev() {

     float ad=0.0;
     float d=0.0;
     float mean=Mean();
 
     for (j=0;j < nset;j++) ad += fabs(d=data[j]-mean);
     	
     return ad/nset;
}     
          

float Moment::Skew() {

     float d=0.0;
     float sk=0.0;
     float mean=Mean();
     for (j=0;j<nset;j++) {
     
          d=data[j]-mean;
	sk += d*d*d;
     }

     if(Var()) {return sk /(nset*Var()*Sdev());}
                            
     else {error("No skew/kurtosis when variance =0 (in moment)");}
}
    
    
float Moment::Kurt() {

     float k=0.0;
     float d=0.0;
     float mean=Mean();
     for (j=0;j < nset;j++) {
              
          d=(data[j]-mean);
          k += d*d*d*d;
     }

     if (Var()) {return (k/(nset*Var()*Var()))-3.0;}
             
     else {error("No skew/kurtosis when variance =0 (in moment)");}     
}

void Moment::error(char ErrorText[]) {

     cerr<<endl<<ErrorText<<endl;
     cout.flush();
     abort();
}     
